'Raymond Allen 2008
'Send a 12-bit picture to the Nokia display

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
      ' Epson S1D15G10 Command Set
      DISON       = $af   
      DISOFF      = $ae   
      DISNOR      = $a6  
      DISINV      = $a7  
      COMSCN      = $bb   
      DISCTL      = $ca   
      SLPIN       = $95   
      SLPOUT      = $94   
      PASET       = $75   
      CASET       = $15   
      DATCTL      = $bc   
      RGBSET8     = $ce   
      RAMWR       = $5c   
      RAMRD       = $5d   
      PTLIN       = $a8   
      PTLOUT      = $a9   
      RMWIN       = $e0   
      RMWOUT      = $ee   
      ASCSET      = $aa   
      SCSTART     = $ab   
      OSCON       = $d1   
      OSCOFF      = $d2   
      PWRCTR      = $20   
      VOLCTR      = $81   
      VOLUP       = $d6   
      VOLDOWN     = $d7   
      TMPGRD      = $82   
      EPCTIN      = $cd   
      EPCOUT      = $cc   
      EPMWR       = $fc   
      EPMRD       = $fd   
      EPSRRD1     = $7c   
      EPSRRD2     = $7d   
      NOPS         = $25  'Appended the "S" to the end of NOP so as not to confuse Prop Tool

  #16, CS, SCLK, SDATA, RST         'Pin assignments:  Using pins 16..19


OBJ
   spi : "SpiEngine"


Var
  byte sx ,sy, ex, ey  'start and ending x&y pixels (depends on rotation!) 
  long bLRInverse, bUDInverse  'image rotation, flip choices

DAT

pic long
file "test1.nok"        'embedded test picture
                        'test1.nok is full screen (130x130) with 1.5 bytes per pixel
                        'test2.nok is smaller with 2 bytes per pixel
      
PUB NokiaTest |i,str

  'Start ASM SPI driver
  spi.start

  'Initialize screen
  initLCD

  SetContrast(20)

  bLRInverse:=false  'make both true to flip display upside down (i.e., rotate 180 degrees)
  bUDInverse:=false  'Note that 1.5 bit bitmaps don't work with UDInverse set

  SetCtlMode($04)  'setup screen based in LR/UD rotation/flip choices, parameter is color mode $4 is 16-bit, 2bytes/pixel 

  'draw a border around visible area to prove display is 130x130
  fillWin(setWin(0,0,129,0),%0000_1111_0000)
  fillWin(setWin(0,0,0,129),%0000_1111_0000)
  fillWin(setWin(129,0,129,129),%0000_1111_0000)
  fillWin(setWin(0,129,129,129),%0000_1111_0000)
  
  'Draw some blue pixels from top-left towards bottom-right to locate top-left corner of screen
  repeat i from 1 to 20
    drawPixel(i,i,%0000_0000_1111)


  'Draw text
  str:=string("Hello")
  repeat i from 0 to strsize(str)-1
    chout(10+16*i,10,byte[str][i])
  str:=string("World")
  repeat i from 0 to strsize(str)-1
    chout(10+16*i,50,byte[str][i])

  'Paint test Bitmap
  PaintBitmap(0,0,@pic)


pub PaintBitmap(x0,y0,pPic)|w,h,area,i,n,mode,a
  'Note:  mode 2 bitmaps (1.5 bytes/pixel) aren't amenable to screen rotation!, need extra work...

  mode:=byte[pPic]
  w:=byte[pPic+1]     'height and width are second&third bytes in data
  h:=byte[pPic+2]

  

  SetCtlMode(mode)  'set color mode 2 (1.5bytes per pixel) or 4 (2 bytes per pixel)


  'set/get fill area
  area:=setWin(x0,y0,x0+w-1,y0+h-1)

  'calculate #bytes
  case mode
    2:
      n:=area*3/2
    4:
      n:=area*2

  'and paint
  spi_Cmd(RAMWR)
  if (mode==2) and  bUDInverse
    'need to flip pixels in pairs
    repeat i from 0 to n-1 step 3
      a:=byte[pPic+3][i]<<16+byte[pPic+3][i+1]<<8+byte[pPic+3][i+2]
      a:=a>>12+a<<12
      spi_Data(a.byte[2])
      spi_Data(a.byte[1])
      spi_Data(a.byte[0])
      
    
  else  'just spit out data
    repeat i from 0 to n-1 
      spi_Data(byte[pPic+3][i])
       
      
pub SetCtlMode(ColorMode)|mode
  'set rotation/flip, rgb arrangement, and 8/16 bit color mode
  mode:=0
  if bUDInverse
    mode+=1
  if bLRInverse
    mode+=2


  spi_Cmd(DATCTL)  ' Data control
  spi_Data(mode) 'this is rotation/flip of the display  
  spi_Data($00) 'This is the RGB arrangement (should always be 0)   
  spi_Data(ColorMode) 'This select 8 or 16-bit color 
  'Set x & y offsets based on mode
  case (mode&$3)
    0: sx:=0
       sy:=2
       
    1: sx:=0
       sy:=0 

    2: sx:=2
       sy:=2 

    3: sx:=2
       sy:=0
  ex:=sx+129  '130 pixels!
  ey:=sy+129  '130 pixels! 

pub SetContrast(i)
      spi_Cmd(VOLCTR)  ' Voltage control
      spi_Data(i)     ' contrast setting: 0..63
      spi_Data(3)      ' resistance ratio (only value that works)

  
PUB drawPixel(x, y, color) 
' Draw a pixel at the specified coordinates and color
'  Assumes colormode $4
' x     =   column address [0..129]
' y     =   row address [0..129]
' color =   16-bit color value: 0000_RRRR_GGGG_BBBB
    x+=sx                ' adjust pixel values based on rotation/flip choices
    y+=sy
    spi_Cmd(CASET)       ' set column address
    spi_Data(x)          ' start
    spi_Data(x)          ' end

    spi_Cmd(PASET)       ' set page (row) address
    spi_Data(y)          ' start
    spi_Data(y)          ' end

    spi_Cmd(RAMWR) 
    spi_Data(color>>8)   ' big endian style
    spi_Data(color)                                                               



Pub fillWin(area, color)| i   ' Fill window with specified color
    spi_Cmd(RAMWR)
    repeat i from 1 to area 
      spi_Data(color>>8)      ' big endian: 0000_RRRR_GGGG_BBBB
      spi_Data(color)



PUB setWin (x0, y0, x1, y1):area | t 
 'Set rectangular area for a window on the LCD
 'Returns its area (in pixels)

    if (x0>x1)
      t:=x0
      x0:=x1
      x1:=t     ' put coordinates into ascending order
    if (y0>y1)
      t:=y0
      y0:=y1
      y1:=t

    spi_Cmd(CASET)       ' set column address
    spi_Data(x0+sx)        ' start (visible area starts in column 2 (when LR inversed))
    spi_Data(x1+sx)        ' end

    spi_Cmd(PASET)        ' set page (row) address
    spi_Data(y0+sy)          ' start
    spi_Data(y1+sy)          ' end

return (x1-x0+1) * (y1-y0+1)   ' area




PUB chOut(penX,penY,ch)|i,j,c,b,s ' Draw ASCII character at penX,penY
    'Modified to use Prop's internal font
    'Assumes color mode 4
    penx+=sx                ' adjust pixel values based on rotation/flip choices
    peny+=sy

    setWin(penX, penY, penX+15, penY+31)
    spi_Cmd(RAMWR)

    repeat j from 0 to 31
      b:= long[$8000+(ch/2)*32*4 +j*4] 
      s:= ch&1
      repeat i from s to 31 step 2 
         c:= 0                              ' black background
         if (b & (1<<i))
            c:= $FFF         ' bright white letters
         spi_Data(c>>8)
         spi_Data(c)


PUB initLcd                     'Initialize the Epson S1D15G00 LCD controller

  DIRA[SCLK]~~  
  DIRA[SDATA]~~
  OUTA[SCLK]~
  OUTA[SDATA]~
  
  'reset display
  DIRA[RST]~~ 
  OUTA[RST]~     ' reset the controller 
  delay_ms(10)
  OUTA[RST]~~
  delay_ms(10)

  DIRA[CS]~~    'enable the chip 
  OUTA[CS]~

  
  spi_Cmd(DISCTL)  ' Display control
  spi_Data(0)      ' 2 divisions, switching period=8 (default)
  spi_Data(32)     ' nlines/4 - 1 = 132/4 - 1 = 32)
  spi_Data(0)      ' no inversely highlighted lines
   
  spi_Cmd(COMSCN)  ' Common scan directions
  spi_Data(1)      ' Scan 1->80, 160<-81
   
  spi_Cmd(OSCON)   ' turn on internal oscillator
   
  spi_Cmd(SLPOUT)  ' Sleep out
   
  spi_Cmd(PWRCTR)  ' Power control
  spi_Data($0F)   ' everything on: reference voltage, regulator,
                  '  circuit voltage follower, and boosters
  spi_Cmd(DISINV)  ' Inverse display (for proper colors)
   
  spi_Cmd(DATCTL)  ' Data control
  spi_Data($02)  'normal mode 3)   ' page address inverted, column address normal,
                  '  address scan in page direction
  spi_Data($00)   ' RGB sequence (default)
  spi_Data($04)   ' ==>  special mode selects 12-bit color for single pixels <==
  'spi_Data($02)
   
  spi_Cmd(VOLCTR)  ' Voltage control
  spi_Data(33)     ' contrast setting: 0..63
  spi_Data(3)      ' resistance ratio (only value that works)
   
  fillWin(setWin(0, 0, 131, 131), 0)  ' clear display memory
   
  delay_MS(100)    ' wait for power supply voltage to stabilize
   
  spi_Cmd(DISON)   ' turn on the display


PRI SPI_Cmd(cmd)
  SpiOutAsm(cmd)

PRI SPI_Data(data)
  SpiOutAsm(data|$100)

PRI SpiOutAsm(Value)
  spi.SHIFTOUT(SDATA, SCLK, 1, 9, Value) 


PRI SPIOUT (Value)| bitNum
'Adapted from Martin Hebel's SHIFTOUT routine from BS2 functions
  outa[SDATA]~             ' Data pin = 0
  dira[SDATA]~~              ' Set data as output
  outa[SCLK]~
  dira[SCLK]~~
  ' Send MSB first               
  REPEAT 9                                                                
     outa[SDATA] := Value >> (9-1)                  ' Set output           
     Value := Value << 1                              ' Shift value right    
     !outa[SCLK]                                      ' cycle clock          
     !outa[SCLK]                                                             
     waitcnt(1000 + cnt)                              ' delay                
  outa[SDATA]~                                            ' Set data to low

PRI delay_ms(milliseconds)
  'pause in milliseconds
  waitcnt(((clkfreq / 1000 * milliseconds - 3932) #> 381) + cnt)       